import type { RequestHandler } from '@sveltejs/kit';
import { Permissions } from '$lib/permissions';
import { deleteAllUserSessions, getSession, updateAllUserSessions } from '$lib/server/session';
import { Admin } from '$lib/server/database';
import { AdminDeleteSchema, AdminEditSchema, AdminListSchema } from './schema';

export const POST = (async ({ request, cookies }) => {
	if (getSession(cookies, { permissions: [Permissions.Admin] }) == null) {
		return new Response(null, { status: 401 });
	}

	const parseResult = await AdminListSchema.safeParseAsync(await request.json());
	if (!parseResult.success) return new Response(null, { status: 400 });
	const data = parseResult.data;

	if (data.username == null || data.password == null || data.permissions == null) {
		return new Response(null, { status: 400 });
	}

	const admin = await Admin.create({
		username: data.username,
		password: data.password,
		permissions: new Permissions(data.permissions)
	});

	delete admin.dataValues.password;

	return new Response(JSON.stringify(admin), { status: 201 });
}) satisfies RequestHandler;

export const PATCH = (async ({ request, cookies }) => {
	if (getSession(cookies, { permissions: [Permissions.Admin] }) == null) {
		return new Response(null, { status: 401 });
	}

	const parseResult = await AdminEditSchema.safeParseAsync(await request.json());
	if (!parseResult.success) return new Response(null, { status: 400 });
	const data = parseResult.data;

	const user = await Admin.findOne({ where: { id: data.id } });
	if (!user) return new Response(null, { status: 400 });

	if (data.username) user.username = data.username;
	if (data.password) user.password = data.password;
	if (data.permissions) user.permissions = new Permissions(data.permissions);
	await user.save();

	updateAllUserSessions(user.id, { permissions: user.permissions });

	return new Response();
}) satisfies RequestHandler;

export const DELETE = (async ({ request, cookies }) => {
	if (getSession(cookies, { permissions: [Permissions.Admin] }) == null) {
		return new Response(null, { status: 401 });
	}

	const parseResult = await AdminDeleteSchema.safeParseAsync(await request.json());
	if (!parseResult.success) return new Response(null, { status: 400 });
	const data = parseResult.data;

	await Admin.destroy({ where: { id: data.id } });
	deleteAllUserSessions(data.id);

	return new Response();
}) satisfies RequestHandler;
